/*
 * Source File < graphpn_.c >
 *
 * n-Dimensional Rotation Port for Borland Graphic Interface
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#include <graphpn_.h>
#include <stdio.h>
#include <malloc.h>
#include <math.h>
#ifdef _XBGI_
#include "xbgidef.h"
#endif

#define hr __HR
extern struct hyperrotationporttype_ hr;

/* rapid n power of 2 */
static
int ipow2( int n )
{
  static int i, first = !0;
  static table[16];

  if (first){
    for ( i=0; i<16; i++ )
      table[i] = 0;
    first = 0;
  }
  if ( n<16 ){
    if ( !table[n] )
      table[n] = (n==0)?1:2*ipow2(n-1);
    return table[n];
  }
  return 1; /* overflow over 16 */
}
#define NR_END 1
/* allocate a double matrix with subscript range m[nrl..nrh][ncl..nch] */
static
double **matrix( long nrl, long nrh, long ncl, long nch )
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  double **m, **tmp;

  /* allocate pointers to rows */
  tmp = m = (double**)malloc( (nrow+NR_END)*sizeof(double*) );
  if ( !m )
    return NULL;
  m += NR_END;
  m -= nrl;
  /* allocate rows and set pointers to them */
  m[nrl] = (double*)malloc( (nrow*ncol+NR_END)*sizeof(double) );
  if ( !m[nrl] ){
    free( tmp );
    return NULL;
  }
  m[nrl] += NR_END;
  m[nrl] -= ncl;
  for ( i=nrl+1; i<=nrh; i++ )
    m[i] = m[i - 1] + ncol;
  /* return pointer to array of pointers to rows */
  return m;
}
/* free a double matrix allocated by matrix() */
static
void free_matrix( double **m, long nrl, long nrh, long ncl, long nch )
{
  free( m[nrl] + ncl - NR_END );
  free( m + nrl - NR_END );
}
/* allocate a int matrix with subscript range m[nrl..nrh][ncl..nch] */
static
int **imatrix(long nrl, long nrh, long ncl, long nch)
{
  long i, nrow = nrh - nrl + 1, ncol = nch - ncl + 1;
  int **m, **tmp;

  /* allocate pointers to rows */
  tmp = m = (int**)malloc( (nrow+NR_END)*sizeof(int*) );
  if ( !m )
    return NULL;
  m += NR_END;
  m -= nrl;
  /* allocate rows and set pointers to them */
  m[nrl] = (int*)malloc( (nrow*ncol+NR_END)*sizeof(int) );
  if ( !m[nrl] ){
    free( tmp );
    return NULL;
  }
  m[nrl] += NR_END;
  m[nrl] -= ncl;
  for (i = nrl + 1; i <= nrh; i++)
    m[i] = m[i - 1] + ncol;
  /* return pointer to array of pointers to rows */
  return m;
}
/* free an int matrix allocated by imatrix() */
static
void free_imatrix( int **m, long nrl, long nrh, long ncl, long nch )
{
  free( m[nrl] + ncl - NR_END );
  free( m + nrl - NR_END );
}
#undef NR_END

void linepn_( double *x1, double *x2 )
{
  rntov( x1, hr.x );
  rntov( x2, hr.y );
  linep3_( hr.x[0],hr.x[1],hr.x[2], hr.y[0],hr.y[1],hr.y[2] );  
}
void linetopn_( double *x )
{
  rntov( x, hr.x );
  linetop3_( hr.x[0],hr.x[1],hr.x[2] );
}
void putpixelpn_( double *x, int c )
{
  rntov( x, hr.x );
  putpixelp3_( hr.x[0],hr.x[1],hr.x[2], c );
}
void movetopn_( double *x )
{
  rntov( x, hr.x );
  movetop3_( hr.x[0],hr.x[1],hr.x[2] );
}
void outtextxypn_( double *x, char *str )
{
  rntov( x, hr.x );
  outtextxyp3_( hr.x[0],hr.x[1],hr.x[2], str );
}
void cubepn_( double *xf, double *xt, int flg )
{
  int dim, nv, ns, nsd;
  int i, j, k, a, b;
  double **v;
  int **s;

  dim = hr.dim;
  nv = ipow2( dim );
  v = matrix( 0,nv-1, 0,dim-1 );
  nsd = ipow2( dim-1 );
  ns = nsd*dim;
  s = imatrix( 0,nsd*dim-1, 0,2 );
  for ( i=0; i<nv; i++ )
    for ( j=0; j<dim; j++ )
      v[i][j] = (i/ipow2(j)%2)?xt[j]:xf[j];
  for( i=0; i<dim; i++ )
    for( j=0; j<nsd; j++ ){
      a = b = 0;
      for( k=0; k<dim; k++ ){
        a = ((i>k)?(j/ipow2(k)%2):(i==k)?0:(j/ipow2(k-1)%2))+2*a;
        b = ((i>k)?(j/ipow2(k)%2):(i==k)?1:(j/ipow2(k-1)%2))+2*b;
      }
      s[nsd*i+j][0] = a;
      s[nsd*i+j][1] = b;
    }
  for ( i=0; i<dim; i++ )
    for ( j=0; j<nsd; j++ ){
      if ( flg )
        setcolor( i%getmaxcolor()+1 );
      linepn_( v[s[nsd*i+j][0]], v[s[nsd*i+j][1]] );
    }
  free_matrix( v, 0,nv-1, 0,dim-1 );
  free_imatrix( s, 0,nsd*dim-1, 0,2 );
}
void linespn_( int n, double **rnpoints )
{
#ifdef _XBGI_
  XPoint *xpoints = (XPoint *)malloc( n*sizeof(XPoint) );
  int i;
  extern xbgi *XBGI;
  
  for ( i=0; i<n; i++ ){
    rntov( rnpoints[i], hr.x );
    xpoints[i].x = Xx(wvx(p3tox(hr.x[0],hr.x[1],hr.x[2])));
    xpoints[i].y = Xy(wvy(p3toy(hr.x[0],hr.x[1],hr.x[2])));
  }
  XDrawLines( XBGIDisplay, XBGIWins[XBGI->apage], XBGIDrawGC,
	     xpoints, n, CoordModeOrigin );
  free( xpoints );
#else
  int i;

  for ( i=0; i<n-1; i++ )
    linepn_( rnpoints[i], rnpoints[i+1] );
#endif
}
void putpixelspn_( int n, double **rnpoints, int c )
{
#ifdef _XBGI_
  XPoint *xpoints = (XPoint *)malloc( n*sizeof(XPoint) );
  int i;
  extern xbgi *XBGI;
  
  XSetForeground( XBGIDisplay, XBGIPixelGC,
                 XBGIPixels[minmax(0,c,XBGI->palette.size-1)] );
  for ( i=0; i<n; i++ ){
    xpoints[i].x = Xx(wvx(p3tox(hr.x[0],hr.x[1],hr.x[2])));
    xpoints[i].y = Xy(wvy(p3toy(hr.x[0],hr.x[1],hr.x[2])));
  }
  XDrawPoints( XBGIDisplay, XBGIWins[XBGI->apage], XBGIPixelGC,
	      xpoints, n, CoordModeOrigin );
  free( xpoints );
#else
  int i;

  for ( i=0; i<n-1; i++ )
    putpixelpn_( rnpoints[i], rnpoints[i+1] );
#endif
}

#undef hr

