/*
 * Header File < graphix.h >
 *
 * Enhanced Functions: Borland Graphics Interface for Xlib
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#ifndef _GRAPHIX_H_
#define _GRAPHIX_H_

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "coniox.h"
#include "graphics.h"

#ifndef NeedFunctionPrototypes
#  if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
#    define NeedFunctionPrototypes 1
#  else
#    define NeedFunctionPrototypes 0
#  endif
#endif

#ifndef __P
#  if defined(__STDC__) || defined(__cplusplus) || defined(c_plusplus)
#    define __P(p) p
#  else
#    define __P(p)
#  endif
#endif

typedef struct _xbgi {
  int argc;
  char **argv;
  int id;
  int only_ascii;
  int page_pixmap;
  int colornames;
  int gdriver, gmode, gerror;
  int npage, apage, vpage;
  int ncolor, color, bkcolor;
  struct pointtype cp;
  struct viewporttype viewport;
  struct linesettingstype lineset;
  struct textsettingstype textset;
  struct fillsettingstype fillset;
  struct palettetype palette;
  int (*button_press_handler)__P(( int button, int x, int y ));
  int (*button_release_handler)__P(( int button, int x, int y ));
  int (*mouse_motion_handler)__P(( int button_state, int x, int y ));
  int (*page_enter_handler)__P(( int page ));
  int (*page_leave_handler)__P(( int page ));
  int (*key_press_handler)__P(( int key ));
  int (*configure_handler)__P(( void ));
  int (*expose_handler)__P(( void ));
  int (*noevent_handler)__P(( void ));
  void (*driver) __P(( void ));
  void (*_driver)__P(( void ));
  void *in;
  struct _xconio *conio;
  struct _xbgi *next;
} xbgi;

#if defined(__cplusplus) || defined(c_plusplus)
extern "C" {
#endif

extern xbgi *_creategraph(
#if NeedFunctionPrototypes
  int __argc, char *__argv[], char *__pd
#endif
);

extern xbgi *creategraph(
#if NeedFunctionPrototypes
  int *__graphdriver, int *__graphmode, char *__pathtodriver
#endif
);

extern xbgi *thisgraph(
#if NeedFunctionPrototypes
  void
#endif
);

extern void destroygraph(
#if NeedFunctionPrototypes
  xbgi *__this
#endif
);

extern void currentgraph(
#if NeedFunctionPrototypes
  xbgi *__this
#endif
);

#if defined(__cplusplus) || defined(c_plusplus)
}
#endif

typedef struct _x_xbgi {
  Display *display;
  Colormap colormap;
  unsigned long background;
  unsigned long foreground;
  unsigned long bottom_shadow;
  unsigned long top_shadow;
  GC default_gc;
  XFontStruct *font_struct;
  Window root;
  Drawable *wins;
  GC draw_gc;
  GC fill_gc;
  GC pixel_gc;
  GC image_gc;
  XFontStruct *stroke_fonts[MAX_CHAR_FONT+1][MAX_CHAR_SIZE+1];
  XFontStruct *bi_fonts[MAX_CHAR_SIZE+1];
  XFontSet multi_fonts[MAX_CHAR_SIZE+1];
  unsigned char colortable[256];
  unsigned long pixels[MAXCOLORS+1];
  XColor rgb[MAXCOLORS+1];
  XEvent event;
} x_xbgi;

/*
 * Inheritor of xbgi for X
 */
#define ixDisplay(it)      (((x_xbgi*)(it->in))->display)
#define ixScreen(it)       (DefaultScreen(ixDisplay(it)))
#define ixVisual(it)       (DefaultVisual(ixDisplay(it),ixScreen(it)))
#define ixDepth(it)        (DefaultDepth(ixDisplay(it),ixScreen(it)))
#define ixVisualClass(it)  (ixVisual(it)->class)
#define ixColormap(it)     (((x_xbgi*)(it->in))->colormap)
#define ixBackground(it)   (((x_xbgi*)(it->in))->background)
#define ixForeground(it)   (((x_xbgi*)(it->in))->foreground)
#define ixBottomShadow(it) (((x_xbgi*)(it->in))->bottom_shadow)
#define ixTopShadow(it)    (((x_xbgi*)(it->in))->top_shadow)
#define ixDefaultGC(it)    (((x_xbgi*)(it->in))->default_gc)
#define ixFontStruct(it)   (((x_xbgi*)(it->in))->font_struct)
#define ixRoot(it)         (((x_xbgi*)(it->in))->root)
#define ixWins(it)         (((x_xbgi*)(it->in))->wins)
#define ixDrawGC(it)       (((x_xbgi*)(it->in))->draw_gc)
#define ixFillGC(it)       (((x_xbgi*)(it->in))->fill_gc)
#define ixPixelGC(it)      (((x_xbgi*)(it->in))->pixel_gc)
#define ixImageGC(it)      (((x_xbgi*)(it->in))->image_gc)
#define ixStrokeFonts(it)  (((x_xbgi*)(it->in))->stroke_fonts)
#define ixBiFonts(it)      (((x_xbgi*)(it->in))->bi_fonts)
#define ixMultiFonts(it)   (((x_xbgi*)(it->in))->multi_fonts)
#define ixColortable(it)   (((x_xbgi*)(it->in))->colortable)
#define ixPixels(it)       (((x_xbgi*)(it->in))->pixels)
#define ixRGB(it)          (((x_xbgi*)(it->in))->rgb)
#define ixEvent(it)        (((x_xbgi*)(it->in))->event)

/*
 * Inheritor of current xbgi for X
 */
#define XBGIDisplay      (ixDisplay(XBGI))
#define XBGIScreen       (ixScreen(XBGI))
#define XBGIVisual       (ixVisual(XBGI))
#define XBGIDepth        (ixDepth(XBGI))
#define XBGIVisualClass  (ixVisualClass(XBGI))
#define XBGIColormap     (ixColormap(XBGI))
#define XBGIBackground   (ixBackground(XBGI))
#define XBGIForeground   (ixForeground(XBGI))
#define XBGIBottomShadow (ixBottomShadow(XBGI))
#define XBGITopShadow    (ixTopShadow(XBGI))
#define XBGIDefaultGC    (ixDefaultGC(XBGI))
#define XBGIFontStruct   (ixFontStruct(XBGI))
#define XBGIRoot         (ixRoot(XBGI))
#define XBGIWins         (ixWins(XBGI))
#define XBGIDrawGC       (ixDrawGC(XBGI))
#define XBGIFillGC       (ixFillGC(XBGI))
#define XBGIPixelGC      (ixPixelGC(XBGI))
#define XBGIImageGC      (ixImageGC(XBGI))
#define XBGIStrokeFonts  (ixStrokeFonts(XBGI))
#define XBGIBiFonts      (ixBiFonts(XBGI))
#define XBGIMultiFonts   (ixMultiFonts(XBGI))
#define XBGIColortable   (ixColortable(XBGI))
#define XBGIPixels       (ixPixels(XBGI))
#define XBGIRGB          (ixRGB(XBGI))
#define XBGIEvent        (ixEvent(XBGI))

#endif /* _GRAPHIX_H_ */

