/*
 * Routines to analyse command line options < options.h >
 *
 * ޥɥ饤󥪥ץϤΤΥ롼
 *
 * Copyright (c) 1995 Taiji YAMADA
 */
#ifndef _OPTIONS_H_
#define _OPTIONS_H_

/*
 * (̾) set_options   --- ʣΥץϿ롼
 *        set_option    --- ٤ʥץϿ롼
 *        scan_options  --- ʣΥץϥ롼
 *        is_option     --- ޥɥ饤Ϸ̼롼
 *        reset_options --- ΰ襯ꥢ롼
 *
 * ()
 *        #include <options.h>
 *
 *        void set_options( n, options )
 *        int n;                   Ͽ륭ο
 *        option_record options[]; Ͽ륭ʤɤǼ¤Τ
 *
 *        option_complex *set_option( key, ... )
 *        char *key, ... ɽʸȲѰ
 *        
 *        void scan_options( argc, argv )
 *        int argc;     ޥɥ饤ο
 *        char *argv[]; ޥɥ饤
 *
 *        int is_option( key );
 *        char *key; ɽʸ
 *        
 *        int reset_options()
 *        
 * ()
 *        ޥɥ饤󥭡򥪥ץ¤Τ⤷ϼ³Ͽ,
 *        Ͽ줿򥳥ޥɥ饤󤫤Ϥ롼󷲤Ǥ.
 *
 *        set_option ⤷ set_options ǥϿ
 *        scan_options ǲϤѿؤʤɤԤޤ.
 *        ġΥ˴ؤϷ̤ is_option ǼǤޤ.
 *        ޤ, Υ롼ϥݤΤ,
 *        Ѹˤ reset_options ¹ԤƲ.
 *        
 *        () ʲϥץϤΰϢμ³Ǥ.
 *
 *             option_record options[] = {
 *               { "-demo",      &flag   },
 *               { "-bit %d",    &bit    },
 *               { "-weight %f", &weight },
 *             };
 *             set_options( 3, options );
 *             set_option( "-help" );
 *             set_option( "-geom %dx%d", &W, &H );
 *             scan_options( argc, argv );
 *             if ( is_option( "-help" ) )
 *               show_help();
 *             reset_options(); 
 */
/*
 * (NAME) set_options   --- options entry routine
 *        set_option    --- a option entry routine
 *        scan_options  --- options analyze routine
 *        is_option     --- analyze result return routine
 *        reset_options --- clear internal buffer routine
 *
 * (SYNOPSIS)
 *        #include <options.h>
 *
 *        void set_options( n, options )
 *        int n;                   the number of key to entry
 *        option_record options[]; array of struct in which is key and so on
 *
 *        option_complex *set_option( key, ... )
 *        char *key, ... string which includes key and argument list
 *        
 *        void scan_options( argc, argv )
 *        int argc;     the number of command line arguments
 *        char *argv[]; command line argument variable
 *
 *        int is_option( key )
 *        char *key; string which includes key
 *        
 *        int reset_options()
 *        
 * (DESCRIPTION)
 *        These routines are for analyzing command line arguments by
 *        keys entried by 'option_record' structure or the procedure.
 *
 *        First, you entry keys by using 'set_option' or 'set_options'.
 *        Next, you execute 'scan_options' and varuable are setted by keys.
 *        You can get each result of key in analyzing.
 *        Last, please execute 'reset_options' to be free
 *        internal memory area used by these routines.
 *        
 *        (ex) An example of analyzing options is as follows;
 *
 *             option_record options[] = {
 *               { "-demo",      NULL    },
 *               { "-bit %d",    &bit    },
 *               { "-weight %f", &weight },
 *             };
 *             set_options( 3, options );
 *             set_option( "-help" );
 *             set_option( "-geom %dx%d", &W, &H );
 *             scan_options( argc, argv );
 *             if ( is_option( "-help" ) )
 *               show_help();
 *             reset_options(); 
 */

typedef struct {
  char *opt;
  void *var;
} option_record;

typedef struct {
  char  *key;
  char  *fmt;
  int    c;
  void **var;
  int    idx;
} option_complex;

option_complex* set_option( char *key, ... );

void set_options( int n, option_record *options );

void scan_options( int argc, char *argv[] );

int is_option( char *key );

void reset_options( void );

#endif /* _OPTIONS_H_ */
