/*
 * Routines to analyse command line options < options.c >
 *
 * ޥɥ饤󥪥ץϤΤΥ롼
 *
 * Copyright (c) 1995 Taiji YAMADA
 */
#include <stdarg.h>
#include <string.h>
#include <malloc.h>
#include <options.h>

static int _n_options = 0;
static option_complex *_options = NULL;

option_complex *set_option( char *key, ... )
{
  int i, n;
  char opt[64], *ptr;
  option_complex *it;
  va_list argptr;

  n = _n_options;
  _n_options++;
  if ( _options == NULL )
    _options = (option_complex*)malloc( _n_options*sizeof(option_complex) );
  else
    _options = (option_complex*)realloc( _options,
					_n_options*sizeof(option_complex) );
  if ( _options == NULL ) return NULL;
  it = &_options[n];

  strcpy( opt, key );
  if ( (ptr=strchr(opt,'%')) != NULL ){
    it->fmt = (char*)malloc( (strlen(ptr)+1)*sizeof(char) );
    strcpy( it->fmt, ptr );
    *ptr = '\0';
    while ( *(--ptr)==' ' ) *ptr = '\0';
  }else
    it->fmt = NULL;
  it->key = (char*)malloc( (strlen(opt)+1)*sizeof(char) );
  strcpy( it->key, opt );

  it->c = 0;
  it->var = NULL;
  it->idx = 0;
  if ( it->fmt != NULL ){
    ptr = strchr( it->fmt, '%' );
    while ( ptr != NULL ){
      it->c++;
      ptr = strchr( ptr+1, '%' );
    }
    it->var = (void**)malloc( (it->c+1)*sizeof(void*) );
    va_start( argptr, key );
    for ( i=0; i<it->c; i++ ){
      it->var[i] = va_arg( argptr, void* );
    }
    it->var[i] = NULL;
    va_end( argptr );
  }
  return it;
}

void set_options( int n, option_record *options )
{
  int i;
  option_complex *it;

  for ( i=0; i<n; i++ ){
    it = set_option( options[i].opt, options[i].var );
    if ( it->fmt == NULL && options[i].var != NULL ){
      it->c = 1;
      it->var = (void**)malloc( (it->c+1)*sizeof(void*) );
      it->var[0] = options[i].var;
      it->var[1] = NULL;
    }
  }
}

void scan_options( int argc, char *argv[] )
{
#define VLIST10(V) V[0],V[1],V[2],V[3],V[4],V[5],V[6],V[7],V[8],V[9]
  int i, n;
  char buf[256];
  option_complex *it;

  for ( n=0; n<_n_options; n++ ){
    it = &_options[n];
    for ( i=1; i<argc; i++ ){
      if ( strcmp( argv[i], it->key )==0 ){
        it->idx = i;
        break;
      }
    }
    if ( it->fmt != NULL && it->idx != 0 ){
      strcpy( buf, "" );
      i = it->idx;
      while ( ++i < argc ){
        strcat( buf, argv[i] );
        strcat( buf, " " );
      }
      sscanf( buf, it->fmt, VLIST10(it->var) );
    }else if ( it->fmt == NULL && it->idx != 0 && it->c > 0 ){
      *(int*)it->var[0] = !0;
    }
  }
#undef VLIST10
}

int is_option( char *key )
{
  int i, n;
  option_complex *it;

  for ( n=0; n<_n_options; n++ ){
    it = &_options[n];
    if ( strcmp( key, it->key ) == 0 ){
      return it->idx;
    }
  }
  return 0;
}

void reset_options( void )
{
  int i, n;
  option_complex *it;

  for ( n=0; n<_n_options; n++ ){
    it = &_options[n];
    if ( it->c != 0 )
      free( it->var );
    if ( it->fmt != NULL ) free( it->fmt );
    if ( it->key != NULL ) free( it->key );
  }
  free( _options );
  _options = NULL;
  _n_options = 0;
}
