/*
 * Source File < x11/driver.c >
 *
 * Standard Functions: Borland Graphics Interface for Xlib
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#include "_bgi.h"
#include "option.h"

static char *graphics_drivers_name[X11+1] = {
  "DETECT",
  "X11",
};
static char *graphics_mode_name[X11+1][X11COLOR+1] = {
  "", "", "", "", "",
  "X11MONO", "X11SGRAY", "X11GRAY", "X11SCOLOR", "X11COLOR",
};

/*
 * registered enhanced function for graphics driver and mode
 */
char* getdriversname( int driver_number )
{
  switch( driver_number ){
  case DETECT:
  case X11:
    return graphics_drivers_name[driver_number];
  }
  return NULL;
}
int getdrivernumber( char *driver_name )
{
  int i;

  if ( driver_name == NULL )
    return DETECT;
  for ( i=X11; i<=X11; i++ )
    if ( strcmp(graphics_drivers_name[i],driver_name)==0 )
      return i;
  return DETECT;
}
int getmodenumber( char *mode_name )
{
  int i = (XBGI!=NULL)?(XBGI->gdriver):X11, j;

  if ( mode_name == NULL )
    return -1;
  for ( j=X11MONO; j<=X11COLOR; j++ )
    if ( strcmp(graphics_mode_name[minmax(X11,i,X11)][j],mode_name)==0 )
      return j;
  return -1;
}
/*
 * standard functions for graphics driver and mode
 */
char* getdrivername( void )
{
  return getdriversname( XBGI->gdriver );
}
void setgraphmode( int gmode )
{
  XBGI->gmode = gmode;
  switch( XBGI->gmode ){
  case X11MONO:
  case X11SGRAY:
    setmaxcolor( 2 );
    break;
  case X11GRAY:
  case X11SCOLOR:
  case X11COLOR:
    setmaxcolor( XBGI->palette.size );
    break;
  }
}
int getgraphmode( void )
{
  return XBGI->gmode;
}
char* getmodename( int mode_number )
{
  int i = (XBGI!=NULL)?(XBGI->gdriver):X11, j;

  if ( X11MONO <= mode_number && mode_number <= X11COLOR )
    return graphics_mode_name[minmax(X11,i,X11)][mode_number];
  return NULL;
}
int getmaxmode( void )
{
  return X11COLOR;
}
void getmoderange( int graphdriver, int *lomode, int *himode )
{
  NotSupported( "getmoderange()" );
}
void _graphfreemem( void *ptr, unsigned size )
{
  NotSupported( "_graphfreemem()" );
}
void* _graphgetmem( unsigned size )
{
  NotSupported( "_graphgetmem()" );
  return NULL;
}
unsigned setgraphbufsize( unsigned bufsize )
{
  NotSupported( "setgraphbufsize()" );
  return 0;
}
int installuserdriver( char *name, int (*detect)( void ) )
{
  ReservedFuture( "installuserdriver()" );
  return 0;
}
int registerbgidriver( void (*driver)( void ) )
{
  XBGI_CURRENT_DRIVER = driver;
  return grOk;
}
int registerfarbgidriver( void *driver )
{
  XBGI_CURRENT_DRIVER = driver;
  return grOk;
}
void detectgraph( int *graphdriver, int *graphmode )
{
  /*
   * case of detecting before initgraph, always X11 and X11COLOR
   */
  if ( XBGIS == NULL || XBGI->in == NULL ){
    *graphdriver = X11;
    *graphmode   = X11COLOR;
    return;
  }
  /*
   * case of detecting after initgraph, research visual class and depth
   */
  switch( *graphdriver ){
  case DETECT: default:
    *graphdriver = XBGI->gdriver = X11;
    switch ( XBGIVisualClass ){
    case StaticGray:
      *graphmode = XBGI->gmode = X11SGRAY;
      if ( XBGIDepth == 1 ){
        *graphmode = XBGI->gmode = X11MONO;
	XBGI->ncolor = 2;
	XBGI->palette.size = (unsigned char)(XBGI->ncolor-1);	
      }
      break;
    case GrayScale:
      *graphmode = XBGI->gmode = X11GRAY;
      if ( XBGIDepth == 1 ){
        *graphmode = XBGI->gmode = X11MONO;
	XBGI->ncolor = 2;
	XBGI->palette.size = (unsigned char)(XBGI->ncolor-1);	
      }
      break;
    case StaticColor:
      *graphmode = XBGI->gmode = X11SCOLOR;
      break;
    case PseudoColor:
    case TrueColor:
    case DirectColor:
      *graphmode = XBGI->gmode = X11COLOR;
      break;
    }
    break;
  case X11:
    *graphmode = XBGI->gmode;
    /* no check */
    break;
  }
}
/*
 * default special driver, destroyer and creater
 */
void _X11_driver( void )
{
  XUnmapSubwindows( XBGIDisplay, XBGIRoot );
  XUnmapWindow( XBGIDisplay, XBGIRoot );
  XFlush( XBGIDisplay );

  XFreeGC( XBGIDisplay, XBGIDrawGC  );
  XFreeGC( XBGIDisplay, XBGIFillGC  );
  XFreeGC( XBGIDisplay, XBGIPixelGC );
  XFreeGC( XBGIDisplay, XBGIImageGC );

  XFreeColormap( XBGIDisplay, XBGIColormap );
  if ( XBGIFontStruct ) XFreeFont( XBGIDisplay, XBGIFontStruct );

  XDestroySubwindows( XBGIDisplay, XBGIRoot );
  XDestroyWindow( XBGIDisplay, XBGIRoot );

#if 0 /* for that events are remaining */
  XSync( XBGIDisplay, False );
  XCloseDisplay( XBGIDisplay );
#endif
#if 0 /* for segmentation fault */
  free( XBGI->in );
  XBGI->in = NULL;
#endif
}
void X11_driver( void )
{
#include "bitmaps/icons/xbgi_a.xbm"
#include "bitmaps/icons/xbgi_b.xbm"
#include "bitmaps/icons/xbgi_c.xbm"
  struct xbm {
    char *bits;
    int width, height;
  } xbms[] = {
    { xbgi_c_bits, xbgi_c_width, xbgi_c_height },
    { xbgi_b_bits, xbgi_b_width, xbgi_b_height },
    { xbgi_a_bits, xbgi_a_width, xbgi_a_height },
    { NULL, 0, 0 }
  };
  int i, j, c, flag_mask, status;
  XSizeHints def;
  XSetWindowAttributes att;
  XEvent ev;
  XColor fgexact, fgcolor, bgexact, bgcolor, lcolor, dcolor;
  Pixmap bm;
  Status ok;
  extern char BGISCOLORNAME[MAXCOLORS][64];
  int argc = XBGI->argc;
  char **argv = XBGI->argv;

  char displayname[64] = "";
  char fnname[80]      = "lucidasanstypewriter-bold-12";
  char fgname[20]      = "white";
  char bgname[20]      = "gray50";
  char defgeomstr[20]  = "640x400";
  char usrgeomstr[20]  = "640x400";
  char titlename[80]   = "";
  char iconname[80]    = "";
  char iconfile[80]    = "";
  
  XBGI->ncolor  = 16;
  XBGI->color   = XBGI->ncolor-1;
  XBGI->bkcolor = 0;
  XBGI->npage   = 2;
  XBGI->apage   = 0;
  XBGI->vpage   = 0;
  strcpy( titlename, argv[0] );
  strcpy( iconname,  argv[0] );
  XBGI->only_ascii = 0;
  XBGI->colornames = 0;

  /* inherit x of xbgi */
  XBGI->in = (x_xbgi*)malloc( sizeof(x_xbgi) );
  if ( XBGI->in == NULL ) FatalError( "XBGI cannot inherit X of XBGI" );

  /* get display name and initialization */
  scan_option( argc, argv, "-display %s", displayname );
  XBGIDisplay = XOpenDisplay( displayname );
  if ( XBGIDisplay == NULL ) FatalError( "X cannot connect to display" );
  XBGIColormap = DefaultColormap( XBGIDisplay, XBGIScreen );

  /* position, size, gc, font and color settings for work */
  scan_option( argc, argv, "-geometry %s", usrgeomstr );
  scan_option( argc, argv, "-fn %s", fnname );
  scan_option( argc, argv, "-fg %s", fgname );
  scan_option( argc, argv, "-bg %s", bgname );
  XBGIDefaultGC = XCreateGC( XBGIDisplay, RootWindow(XBGIDisplay,XBGIScreen),
			    0, NULL );
  XBGIFontStruct = XLoadQueryFont( XBGIDisplay, fnname );
  if ( XBGIFontStruct == NULL ) Warning( "X cannot find font structure" );
  else
    XSetFont( XBGIDisplay, XBGIDefaultGC, XBGIFontStruct->fid );
  ok = XAllocNamedColor( XBGIDisplay, XBGIColormap, fgname,
                        &fgexact, &fgcolor );
  XBGIForeground = (ok==True)?fgexact.pixel:WhitePixel(XBGIDisplay,XBGIScreen);
  ok = XAllocNamedColor( XBGIDisplay, XBGIColormap, bgname,
                        &bgexact, &bgcolor );
  XBGIBackground = (ok==True)?bgexact.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  lcolor.red   = min( bgexact.red  *5/4, 65535 );
  lcolor.green = min( bgexact.green*5/4, 65535 );
  lcolor.blue  = min( bgexact.blue *5/4, 65535 );
  dcolor.red   = max( 0, bgexact.red  *3/4 );
  dcolor.green = max( 0, bgexact.green*3/4 );
  dcolor.blue  = max( 0, bgexact.blue *3/4 );
  ok = XAllocColor( XBGIDisplay, XBGIColormap, &lcolor );
  XBGITopShadow = (ok==True)?lcolor.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  ok = XAllocColor( XBGIDisplay, XBGIColormap, &dcolor );
  XBGIBottomShadow
    = (ok==True)?dcolor.pixel:BlackPixel(XBGIDisplay,XBGIScreen);
  XSetForeground( XBGIDisplay, XBGIDefaultGC, XBGIForeground );

  /* titlename and iconname settings */
  if ( scan_option( argc, argv, "-title %s", titlename ) )
    strcpy( iconname, titlename );
  scan_option( argc, argv, "-name %s", iconname );
  scan_option( argc, argv, "-icon %s", iconfile );

  /* get user bgi settings */
  if ( (i=scan_option(argc,argv,"-gd")) )
    XBGI->gdriver = getdrivernumber( argv[i+1] );
  if ( (i=scan_option(argc,argv,"-gm")) )
    XBGI->gmode = getmodenumber( argv[i+1] );
  scan_option( argc, argv, "-ncolor %d",  &(XBGI->ncolor)  );
  scan_option( argc, argv, "-color %d",   &(XBGI->color)   );
  scan_option( argc, argv, "-bkcolor %d", &(XBGI->bkcolor) );
  scan_option( argc, argv, "-npage %d",   &(XBGI->npage)   );
  scan_option( argc, argv, "-apage %d",   &(XBGI->apage)   );
  scan_option( argc, argv, "-vpage %d",   &(XBGI->vpage)   );

  /* scanning named color list */
  if ( (j=scan_option(argc,argv,"-colornames")) ){
    c = 0;
    for ( i=j+1; i<argc; i++ ){
      if ( *argv[i] == '-' )
        break;
      strcpy( BGISCOLORNAME[c++], argv[i] );
    }
    if ( 2 <= c && c<256 ){
      XBGI->colornames = !0;
      XBGI->ncolor = c;
    }
  }
  XBGI->palette.size = (unsigned char)(XBGI->ncolor-1);

  /* set font valuable */
  if ( scan_option( argc, argv, "-only-ascii"  ) ) XBGI->only_ascii = !0;
  if ( scan_option( argc, argv, "-only-ascii-" ) ) XBGI->only_ascii =  0;
#ifdef ENABLE_MULTILINGUAL
  if ( setlocale( LC_ALL, "" ) == NULL )
    XBGI->only_ascii = !0;
  else if ( XSupportsLocale() == False )
    XBGI->only_ascii = !0;
  if ( !XBGI->only_ascii )
    XSetLocaleModifiers( "" );
#endif

  /* page characteristics */
  if ( scan_option( argc, argv, "-page-pixmap"  ) ) XBGI->page_pixmap = !0;
  if ( scan_option( argc, argv, "-page-pixmap-" ) ) XBGI->page_pixmap =  0;

  /* create windows */
  flag_mask = XGeometry( XBGIDisplay, XBGIScreen, usrgeomstr, defgeomstr,
                        0, 1, 1, 0, 0,
                        &def.x, &def.y, &def.width, &def.height );
  XBGIRoot = XCreateSimpleWindow( XBGIDisplay,
				 RootWindow(XBGIDisplay,XBGIScreen),
				 def.x, def.y, def.width, def.height, 0,
				 WhitePixel(XBGIDisplay,XBGIScreen),
				 BlackPixel(XBGIDisplay,XBGIScreen) );
  def.min_width  = 1;
  def.min_height = 1;
  def.max_width  = 2000;
  def.max_height = 2000;
  def.flags = PMinSize | PMaxSize;
  if ( flag_mask&XValue )      def.flags = def.flags | PPosition;
  if ( flag_mask&YValue )      def.flags = def.flags | USPosition;
  if ( flag_mask&WidthValue )  def.flags = def.flags | PSize;
  if ( flag_mask&HeightValue ) def.flags = def.flags | USSize;
  XSelectInput( XBGIDisplay, XBGIRoot, ExposureMask|StructureNotifyMask );
  XSetStandardProperties( XBGIDisplay, XBGIRoot,
                         titlename, iconname, None, argv, argc, &def );
  XSetWMNormalHints( XBGIDisplay, XBGIRoot, &def );

  /* input event selection, GC creation and so on */
  XBGIDepth   = DefaultDepth( XBGIDisplay, XBGIScreen );
  XBGIDrawGC  = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIFillGC  = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIPixelGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  XBGIImageGC = XCreateGC( XBGIDisplay, XBGIRoot, 0, NULL );
  att.backing_store = Always;
  if ( scan_option( argc, argv, "-backing"  ) )
    att.backing_store = Always;
  if ( scan_option( argc, argv, "-backing-" ) )
    att.backing_store = NotUseful;
  XBGIWins = (Window*)malloc( XBGI->npage*sizeof(Window) );
  for ( i=0; i<XBGI->npage; i++ ){
    XBGIWins[i] = XCreateSimpleWindow( XBGIDisplay, XBGIRoot, 0, 0,
                                       def.width, def.height, 0,
                                       WhitePixel(XBGIDisplay,XBGIScreen),
                                       BlackPixel(XBGIDisplay,XBGIScreen) );
    XSelectInput( XBGIDisplay, XBGIWins[i], ExposureMask|
                 OwnerGrabButtonMask|ButtonPressMask|ButtonReleaseMask|
                 PointerMotionMask|ButtonMotionMask|KeyPressMask|
                 EnterWindowMask|LeaveWindowMask );
    XChangeWindowAttributes( XBGIDisplay, XBGIWins[i], CWBackingStore, &att );
    XSetStandardProperties( XBGIDisplay, XBGIWins[i],
                           titlename, iconname, None, argv, argc, &def );
  }
  
  /* initializing fonts */
  for ( i=DEFAULT_FONT; i<=MAX_CHAR_FONT; i++ )
    for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
      XBGIStrokeFonts[i][j] = NULL;
#ifdef ENABLE_BILINGUAL
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIBiFonts[j] = NULL;
#endif
#ifdef ENABLE_MULTILINGUAL
  for ( j=USER_CHAR_SIZE; j<=MAX_CHAR_SIZE; j++ )
    XBGIMultiFonts[j] = NULL;
#endif

  /* defualt event procedures in xbgiloop */
  XBGI->button_press_handler   = _x_xbgi_button_press;
  XBGI->button_release_handler = _x_xbgi_button_release;
  XBGI->mouse_motion_handler   = _x_xbgi_mouse_motion;
  XBGI->page_enter_handler     = _x_xbgi_page_enter;
  XBGI->page_leave_handler     = _x_xbgi_page_leave;
  XBGI->key_press_handler      = _x_xbgi_key_press;
  XBGI->configure_handler      = _x_xbgi_configure;
  XBGI->expose_handler         = _x_xbgi_expose;
  XBGI->noevent_handler        = _x_xbgi_noevent;

  /* BGI default setting */
  detectgraph( &(XBGI->gdriver), &(XBGI->gmode) );
  setgraphmode( XBGI->gmode );
  graphdefaults();

  XMapWindow( XBGIDisplay, XBGIRoot );
  XMapSubwindows( XBGIDisplay, XBGIRoot ); 
  do { /* bug fixed at version 2.3 */
    XNextEvent( XBGIDisplay, &ev );
  } while ( Expose != ev.type );
  XFlush( XBGIDisplay );

  /* set icon from data or file */
  if ( strcmp(iconfile,"")==0 ){
    for ( i=0; xbms[i].bits != NULL; i++ ){
      bm = XCreateBitmapFromData( XBGIDisplay,
				 RootWindow(XBGIDisplay,XBGIScreen),
                                 xbms[i].bits, xbms[i].width, xbms[i].height );
      status = XSetIconFromPixmap( XBGIDisplay, XBGIRoot, bm,
                                  (unsigned int)xbms[i].width,
                                  (unsigned int)xbms[i].height );
      XFreePixmap( XBGIDisplay, bm );
      if ( status == True )
	break;
    }
  }else{
    seticon( iconfile );
  }

  XBGI->gerror = grOk;
  XBGI->_driver = _X11_driver;
}

int X11_driver_far[] = { 999 }; /* dummy */


