/*
 * Source File < xbgiutl.c >
 *
 * XBGI Utilities Functions: Borland Graphics Interface for Xlib
 *
 * Copyright (c) 1992-94 Tokyo Denki University, Taiji Yamada
 */
#include "_bgi.h"

int XSetIconFromPixmap( Display *d, Window r,
                       Pixmap pm, unsigned int w, unsigned int h )
{
  int i, j, n;
  XIconSize *is;
  int status;
  Bool wok = False, hok = False;
  XWMHints wmh;
  
  status = XGetIconSizes( d, r, &is, &n );
  if ( status == 0 ) return 0;
  for ( i=0; i<n; i++ ){
    for ( j=is[i].min_width; j<=is[i].max_width; j+=is[i].width_inc )
      if ( w == j ){
        wok = True;
        break;
      }
    for ( j=is[i].min_height; j<=is[i].max_height; j+=is[i].height_inc )
      if ( h == j ){
        hok = True;
        break;
      }
  }
  if ( wok && hok ){
    wmh.flags = IconPixmapHint;
    wmh.icon_pixmap = pm;
    XSetWMHints( d, r, &wmh );
    return !0;
  }else{
    return 0;
  }
}
void xbgiGetSizeVPrintf( XFontStruct *fs, int *width, int *height,
                        char *fmt, va_list argptr )
{
  int i, maxwidth, c = 0; 
  char str[1024], *ptr;
  
  vsprintf( str, fmt, argptr );
  strcat( str, "\n" );
  ptr = strtok( str, "\n" );
  while ( ptr != NULL ){
    c++;
    ptr = strtok( NULL, "\n" );
  }
  ptr = str;
  *height = TextHeight(fs);
  maxwidth = TextWidth( fs, ptr, strlen(ptr) );
  for ( i=0; i<c; i++ ){
    maxwidth = max( maxwidth, TextWidth( fs,ptr,strlen(ptr) ) );
    ptr = ptr + strlen(ptr);
    while ( ptr++ == NULL );
  }
  *width  = Brim(maxwidth);
  *height = Brim(*height)*c;
}
void xbgiGetSizePrintf( XFontStruct *fs, int *width, int *height, char *fmt,
                       ... )
{
  va_list argptr;
  
  va_start( argptr, fmt );
  xbgiGetSizeVPrintf( fs, width, height, fmt, argptr );
  va_end( argptr );
}
void xbgiVPrintf( Display *display, Window win, GC gc, XFontStruct *fs,
                 int textjust, char *fmt, va_list argptr )
{
  Window root;
  int x, y, width, height, border, depth;
  int i, c = 0, textx, texty;
  char str[1024], *ptr;
  
  vsprintf( str, fmt, argptr );
  strcat( str, "\n" );
  ptr = strtok( str, "\n" );
  while ( ptr != NULL ){
    c++;
    ptr = strtok( NULL, "\n" );
  }
  XGetGeometry( display, win, &root, &x, &y,
               &width, &height, &border, &depth );
  ptr = str;
  for ( i=0; i<c; i++ ){
    switch( textjust ){
    case LEFT_TEXT:
      textx = HemBrim;
      break;
    case CENTER_TEXT:
      textx = ( width - TextWidth(fs,ptr,strlen(ptr)) )/2;
      break;
    case RIGHT_TEXT:
      textx = width - HemBrim - TextWidth(fs,ptr,strlen(ptr));
      break;
    }
    texty = HemBrim + TextAscent(fs) + i*Brim(TextHeight(fs));
    XDrawString( display, win, gc, textx, texty, ptr, strlen(ptr) );
    ptr = ptr + strlen(ptr);
    while ( ptr++ == NULL );
  }
}
void xbgiPrintf( Display *display, Window win, GC gc, XFontStruct *fs,
                int textjust, char *fmt, ... )
{
  va_list argptr;

  va_start( argptr, fmt );
  xbgiVPrintf( display, win, gc, fs, textjust, fmt, argptr );
  va_end( argptr );
}
void xbgiDecorateWindow( Display *display, Window win, GC gc,
                        unsigned long fg, unsigned long lc, unsigned long dc,
                        int type )
{
  XPoint orec[4], irec[4], lrec[4], trec[4], rrec[4], brec[4];
  XWindowAttributes att;

  XGetWindowAttributes( display, win, &att );
  orec[0].x = 0;                     orec[0].y = 0; 
  orec[1].x = 0;                     orec[1].y = att.height; 
  orec[2].x = att.width;             orec[2].y = att.height; 
  orec[3].x = att.width;             orec[3].y = 0; 
  irec[0].x = DecoHemBits;           irec[0].y = DecoHemBits; 
  irec[1].x = DecoHemBits;           irec[1].y = att.height-DecoHemBits; 
  irec[2].x = att.width-DecoHemBits; irec[2].y = att.height-DecoHemBits; 
  irec[3].x = att.width-DecoHemBits; irec[3].y = DecoHemBits; 
  lrec[0] = orec[0]; lrec[1] = orec[1]; lrec[2] = irec[1]; lrec[3] = irec[0];
  trec[0] = orec[0]; trec[1] = orec[3]; trec[2] = irec[3]; trec[3] = irec[0];
  rrec[0] = orec[2]; rrec[1] = orec[3]; rrec[2] = irec[3]; rrec[3] = irec[2];
  brec[0] = orec[1]; brec[1] = orec[2]; brec[2] = irec[2]; brec[3] = irec[1];
  XClearWindow( display, win );
  switch( type ){
  case DecoNothing:
    break;
  case DecoRectFront: 
    XSetForeground( display, gc, lc );
    XFillPolygon( display, win, gc, lrec, 4, Complex, CoordModeOrigin );
    XFillPolygon( display, win, gc, trec, 4, Complex, CoordModeOrigin );
    XSetForeground( display, gc, dc );
    XFillPolygon( display, win, gc, rrec, 4, Complex, CoordModeOrigin );
    XFillPolygon( display, win, gc, brec, 4, Complex, CoordModeOrigin );
    break;
  case DecoRectBack: 
    XSetForeground( display, gc, dc );
    XFillPolygon( display, win, gc, lrec, 4, Complex, CoordModeOrigin );
    XFillPolygon( display, win, gc, trec, 4, Complex, CoordModeOrigin );
    XSetForeground( display, gc, lc );
    XFillPolygon( display, win, gc, rrec, 4, Complex, CoordModeOrigin );
    XFillPolygon( display, win, gc, brec, 4, Complex, CoordModeOrigin );
    break;
  }
  XSetForeground( display, gc, fg );
}



